==> gen_stage-0.6.1.tar: lib/gen_stage/dispatcher.ex <==
alias Experimental.GenStage

defmodule GenStage.Dispatcher do
  @moduledoc """
  This module defines the behaviour used by `:producer` and
  `:producer_consumer` to dispatch events.

  Elixir ships with two dispatcher implementations:

    * `GenStage.DemandDispatcher` - dispatches the given batch of
      events to the consumer with the biggest demand in a FIFO
      ordering. This is the default dispatcher.

    * `GenStage.BroadcastDispatcher` - dispatches all events to all
      consumers. The demand is only sent upstream once all consumers
      ask for data.
  """

  @doc """
  Called on initialization with the options given on `c:GenStage.init/1`.
  """
  @callback init(opts :: keyword()) :: {:ok, state} when state: any

  @doc """
  Called every time the producer gets a new subscriber.
  """
  @callback subscribe(opts :: keyword(), from :: {pid, reference}, state :: term) ::
    {:ok, demand :: non_neg_integer, new_state} when new_state: term

  @doc """
  Called every time a subscription is cancelled or the consumer goes down.

  It is guaranteed the reference given in `from` points to a reference
  previously given in subscribe.
  """
  @callback cancel(from :: {pid, reference}, state :: term) ::
    {:ok, demand :: non_neg_integer, new_state} when new_state: term

  @doc """
  Called every time a consumer sends demand.

  The demand will always be a positive integer (more than 0).
  This callback must return the `actual_demand` as part of its
  return tuple. The returned demand is then sent to producers.

  It is guaranteed the reference given in `from` points to a
  reference previously given in subscribe.
  """
  @callback ask(demand :: pos_integer, from :: {pid, reference}, state :: term) ::
    {:ok, actual_demand :: non_neg_integer, new_state} when new_state: term

  @doc """
  Called every time a producer wants to dispatch an event.

  The events will always be a non empty list. This callback may
  receive more events than previously asked and therefore must
  return events it cannot not effectively deliver as part of its
  return tuple. Any `leftover_events` will be stored by producers
  in their buffer.

  It is important to emphasize that `leftover_events` can happen
  in any dispatcher implementation. After all, a consumer can
  subscribe, ask for events and crash. Eventually the events
  the consumer asked will be delivered while the consumer no longer
  exists, meaning they must be returned as left_over events until
  another consumer subscribes.

  It is guaranteed the reference given in `from` points to a
  reference previously given in subscribe. It is also recommended
  for events to be sent with `Process.send/3` and the `[:noconnect]`
  option as the consumers are all monitored by the producer. For
  example:

      Process.send(consumer, {:"$gen_consumer, {self(), consumer_ref}, events}, [:noconnect])

  """
  @callback dispatch(events :: nonempty_list(term), state :: term) ::
    {:ok, leftover_events :: [term], new_state} when new_state: term

  @doc """
  Used to send a notification to all consumers.

  In case the dispatcher is doing buffering, notify must keep
  the ordering guarantees. It is recommended for the notification
  to be sent with `Process.send/3` and the `[:noconnect]` option
  as the consumers are all monitored by the producer. For example:

      Process.send(consumer, {consumer_ref, msg}, [:noconnect])
  """
  @callback notify(msg :: term, state :: term) ::
    {:ok, new_state} when new_state: term
end

==> gen_stage-0.8.0.tar: lib/gen_stage/dispatcher.ex <==
alias Experimental.GenStage

defmodule GenStage.Dispatcher do
  @moduledoc """
  This module defines the behaviour used by `:producer` and
  `:producer_consumer` to dispatch events.

  Elixir ships with two dispatcher implementations:

    * `GenStage.DemandDispatcher` - dispatches the given batch of
      events to the consumer with the biggest demand in a FIFO
      ordering. This is the default dispatcher.

    * `GenStage.BroadcastDispatcher` - dispatches all events to all
      consumers. The demand is only sent upstream once all consumers
      ask for data.
  """

  @doc """
  Called on initialization with the options given on `c:GenStage.init/1`.
  """
  @callback init(opts :: keyword()) :: {:ok, state} when state: any

  @doc """
  Called every time the producer gets a new subscriber.
  """
  @callback subscribe(opts :: keyword(), from :: {pid, reference}, state :: term) ::
    {:ok, demand :: non_neg_integer, new_state} when new_state: term

  @doc """
  Called every time a subscription is cancelled or the consumer goes down.

  It is guaranteed the reference given in `from` points to a reference
  previously given in subscribe.
  """
  @callback cancel(from :: {pid, reference}, state :: term) ::
    {:ok, demand :: non_neg_integer, new_state} when new_state: term

  @doc """
  Called every time a consumer sends demand.

  The demand will always be a positive integer (more than 0).
  This callback must return the `actual_demand` as part of its
  return tuple. The returned demand is then sent to producers.

  It is guaranteed the reference given in `from` points to a
  reference previously given in subscribe.
  """
  @callback ask(demand :: pos_integer, from :: {pid, reference}, state :: term) ::
    {:ok, actual_demand :: non_neg_integer, new_state} when new_state: term

  @doc """
  Called every time a producer wants to dispatch an event.

  The events will always be a non empty list. This callback may
  receive more events than previously asked and therefore must
  return events it cannot not effectively deliver as part of its
  return tuple. Any `leftover_events` will be stored by producers
  in their buffer.

  It is important to emphasize that `leftover_events` can happen
  in any dispatcher implementation. After all, a consumer can
  subscribe, ask for events and crash. Eventually the events
  the consumer asked will be delivered while the consumer no longer
  exists, meaning they must be returned as left_over events until
  another consumer subscribes.

  It is guaranteed the reference given in `from` points to a
  reference previously given in subscribe. It is also recommended
  for events to be sent with `Process.send/3` and the `[:noconnect]`
  option as the consumers are all monitored by the producer. For
  example:

      Process.send(consumer, {:"$gen_consumer, {self(), consumer_ref}, events}, [:noconnect])

  """
  @callback dispatch(events :: nonempty_list(term), length :: pos_integer, state :: term) ::
    {:ok, leftover_events :: [term], new_state} when new_state: term

  @doc """
  Used to send a notification to all consumers.

  In case the dispatcher is doing buffering, notify must keep
  the ordering guarantees. It is recommended for the notification
  to be sent with `Process.send/3` and the `[:noconnect]` option
  as the consumers are all monitored by the producer. For example:

      Process.send(consumer, {consumer_ref, msg}, [:noconnect])
  """
  @callback notify(msg :: term, state :: term) ::
    {:ok, new_state} when new_state: term
end
